package vplotter.model;

import java.util.ArrayList;
import java.util.List;
/**
 * Baseado em http://www.math.ucla.edu/~baker/java/hoefer/Bezier.htm
 * 
 * @author leonardootto
 * 
 */
public class Bezier {

	private Point[] coordlist;
	double t; // the time interval
	double k = .025; // time step value for drawing curve

	public Bezier(Point startPoint, Point endPoint, Point controlStart, Point controlEnd) {
		super();
		this.coordlist = new Point[4];
		this.coordlist[0] = startPoint;
		this.coordlist[1] = controlStart;
		this.coordlist[2] = controlEnd;
		this.coordlist[3] = endPoint;
	}
	
	public List<Line> getLines() {
		List<Line> lines = new ArrayList<Line>();

		double x1, x2, y1, y2;
		x1 = coordlist[0].x;
		y1 = coordlist[0].y;
		for (t = k; t <= 1 + k; t += k) {
			// use Berstein polynomials
			x2 = (coordlist[0].x + t
					* (-coordlist[0].x * 3 + t
							* (3 * coordlist[0].x - coordlist[0].x * t)))
					+ t
					* (3 * coordlist[1].x + t
							* (-6 * coordlist[1].x + coordlist[1].x * 3 * t))
					+ t
					* t
					* (coordlist[2].x * 3 - coordlist[2].x * 3 * t)
					+ coordlist[3].x * t * t * t;
			y2 = (coordlist[0].y + t
					* (-coordlist[0].y * 3 + t
							* (3 * coordlist[0].y - coordlist[0].y * t)))
					+ t
					* (3 * coordlist[1].y + t
							* (-6 * coordlist[1].y + coordlist[1].y * 3 * t))
					+ t
					* t
					* (coordlist[2].y * 3 - coordlist[2].y * 3 * t)
					+ coordlist[3].y * t * t * t;
			//add lines
			lines.add(new Line(new Point(x1, y1), new Point(x2, y2)));
			// offscreenG.drawLine((int) x1, (int) y1, (int) x2, (int) y2);
			x1 = x2;
			y1 = y2;
		}
		return lines;
	}
}
